/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

/**
 * Evaluation results.
 */

#ifndef _INPUTOUTPUT_H
#define _INPUTOUTPUT_H

#if (defined _MSC_VER) && (!defined snprintf)
#define snprintf _snprintf
#endif

inline void
SaveDiscrepancyResults( const int nbResults, std::string *featureNames, DiscrepancyResult *results, Collection &cold ) {
   const int nbIndicators= results[0].size;
   Float** valueArray = new Float*[nbIndicators * 2];
   Float** normalizationArray = new Float*[nbIndicators * 2];
   
   for (int i = 0; i < nbIndicators; i++) {
      valueArray[i * 2] = new Float[nbResults];
      normalizationArray[i * 2] = new Float[nbResults];
      valueArray[i * 2 + 1] = new Float[nbResults];
      normalizationArray[i * 2 + 1] = new Float[nbResults];
      for (int j = 0; j < nbResults; j++) {
       	 valueArray[i * 2][j] = results[j].indicators[i].errors[0].value;
      	 normalizationArray[i * 2][j] = results[j].indicators[i].errors[0].normalization;
      	 valueArray[i * 2 + 1][j] = results[j].indicators[i].errors[1].value;
      	 normalizationArray[i * 2 + 1][j] = results[j].indicators[i].errors[1].normalization;
      }
   }
   
   char tmp[256];
   for (int i = 0; i < nbIndicators * 2 ; i++) {
      snprintf(tmp, sizeof(tmp), "%s.%d", featureNames[0].c_str(), i + 1);
      tmp[sizeof(tmp) - 1] = 0;
      cold.SETARRAY(tmp, Float, valueArray[i], nbResults);
      snprintf(tmp, sizeof(tmp), "%s.%d", featureNames[1].c_str(), i + 1);
      tmp[sizeof(tmp) - 1] = 0;
      cold.SETARRAY(tmp, Float, normalizationArray[i], nbResults);
   }
}

inline void
SaveEvaluationResult( const EvaluationResult & evaluationError, Collection & cold, const std::string errorName ) {
   char tmp[256];
   for (int i = 0; i < evaluationError.size; i++ ) {
      snprintf(tmp, sizeof(tmp), "%s.%d", errorName.c_str(), i + 1);
      tmp[sizeof(tmp) - 1] = 0;
      cold.SETVALUE(tmp, Float, evaluationError.errors[i]);   
   }
}

inline void
SaveEvaluationResult( const EvaluationResult & evaluationError, Collection & cold, const std::string errorNames[] ) {
   for (int i = 0; i < evaluationError.size; i++ ) {
      cold.SETVALUE(errorNames[i], Float, evaluationError.errors[i]);   
   }
}

#endif
