/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 pra *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

#ifdef MAIN
#undef MAIN
#include "pranksegmentationalgorithms.cpp"
#include "./evaluation/evaluatealgorithm.h"
#include "./evaluation/computeerrorvalues.h"
#define MAIN
#else
#include "pranksegmentationalgorithms.cpp"
#include "./evaluation/evaluatealgorithm.h"
#include "./evaluation/computeerrorvalues.h"
#endif

#include <vector>

Errc
PRankSegmentationAlgorithmsFromFolders( int matching_algorithm_id, float matching_threshold, 
					int acceptable_error1, int priority1,
					int acceptable_error2, int priority2,
					int acceptable_error3, int priority3,
					int acceptable_error4, int priority4,
					int acceptable_error5, int priority5 ,
					std::string segmentation_result_path, 
					std::string ground_truth_path, 
					Collection & cold1, 
					Collection & cold2 ) {
   printf("gt = %s\n", ground_truth_path.c_str());
   std::vector<std::string> files = std::vector<std::string>();
   std::vector<std::string> directories = std::vector<std::string>();
   
   if (!endsWith(segmentation_result_path, PATH_SEPARATOR)) {
      segmentation_result_path += PATH_SEPARATOR;
   }
   if (!endsWith(ground_truth_path, PATH_SEPARATOR)) {
      ground_truth_path += PATH_SEPARATOR;
   }
   
   if (!isRegularDirectory(segmentation_result_path)) {
      LOG(INFO) << "[ERROR: " << errno << " ] Couldn't open folder " << segmentation_result_path << "." << std::endl;
      return FAILURE;
   }
   if (!isRegularDirectory(ground_truth_path)) {
      LOG(INFO) << "[ERROR: " << errno << " ] Couldn't open folder " << ground_truth_path << "." << std::endl;
      return FAILURE;
   }
    
   getSubdirectories(segmentation_result_path, directories);
    
   std::sort(files.begin(), files.end());
   std::sort(directories.begin(), directories.end() );
   Collection coli1;
   Collection coli2;
   Collection coli3;

   std::vector<std::string> algorithm_names;
   const int nbOfAlgorithms = directories.size();
   const Long nbOfIndicators = 5;
   EvaluationPerformance *performances = new EvaluationPerformance[nbOfAlgorithms];

   for (unsigned int algorithm = 0; algorithm < directories.size(); algorithm++) {
      std::string algorithm_name = basename(directories[algorithm]);
      LOG(INFO)  << " Algorithm:  " << algorithm_name << std::endl;
      algorithm_names.push_back(algorithm_name);
      std::string resultdir = dirname(directories[algorithm]);
      LOG(INFO)  << " result path: " << resultdir << std::endl;
      LOG(INFO)  << " ground truth path: " <<  ground_truth_path << std::endl;

      DiscrepancyResult * discrepancyErrors;
      int nbImages = EvaluateAlgorithm(matching_algorithm_id, matching_threshold,  resultdir + algorithm_name, ground_truth_path, discrepancyErrors);
      if (nbImages > 0) {
	 const int nbIndicators = 5;
	 EvaluationResult evaluationResult(nbIndicators * 2);
	 ComputeErrorValues(discrepancyErrors, nbImages, evaluationResult);
	 performances[algorithm] = ComputeIndicators(evaluationResult, acceptable_error1, acceptable_error2, acceptable_error3, acceptable_error4, acceptable_error5);
      }
   }
   
   Long *ranks = new Long[nbOfAlgorithms];
  
   int priority[5];
   priority[0] = priority1;
   priority[1] = priority2;
   priority[2] = priority3;
   priority[3] = priority4;
   priority[4] = priority5;
   MultiIndicatorRanking(performances, ranks, nbOfAlgorithms, nbOfIndicators, priority);

   displayIndicators(performances, nbOfAlgorithms, nbOfIndicators);
   displayRanks(ranks, nbOfAlgorithms);

   buildOutputResult1(performances, cold1, nbOfAlgorithms, nbOfIndicators);
   buildOutputResult2(ranks, cold2, nbOfAlgorithms);

   return SUCCESS;
}

#ifdef MAIN

/*
 * Modify only the following constants, and the function call.
 */
#define	USAGE	"usage: %s matching_algorithm_id matching_threshold acceptable_error1 priority1 acceptable_error2 priority2 acceptable_error3 priority3 acceptable_error4 priority4 acceptable_error5 priority5 segmentation_result_path ground_truth_path indicator_collection rank_collection"

#define	PARC	14
#define	FINC	0
#define	FOUTC	2
#define	MASK	0

int
main( int argc, char *argv[] ) {
   Errc  result;		// The result code of the execution.
   Pobject* mask;		// The region mask.
   Pobject* objin[FINC + 1];	// The input objects;
   Pobject* objs[FINC+  1];	// The source objects masked by the mask.
   Pobject* objout[FOUTC + 1];	// The output object.
   Pobject* objd[FOUTC + 1];	// The result object of the execution.
   char* parv[PARC + 1];	// The input parameters.

   if (!strcmp(argv[1], "-v")) {
      setVerbose();
      argc --;
      argv[1] = argv[0];
      argv++;
   }

   ReadArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, parv, USAGE, MASK);
   
   objd[0] = new Collection;
   Collection* cold1 = (Collection*)objd[0];
   objd[1] = new Collection;
   Collection* cold2 = (Collection*)objd[1];

   result = PRankSegmentationAlgorithmsFromFolders(atoi(parv[0]), (float)atof(parv[1]), 
						   atoi(parv[2]), atoi(parv[3]), 
						   atoi(parv[4]), atoi(parv[5]), 
						   atoi(parv[6]), atoi(parv[7]), 
						   atoi(parv[8]), atoi(parv[9]), 
						   atoi(parv[10]), atoi(parv[11]), 
						   std::string(parv[12]), std::string(parv[13]), 
						   *cold1, *cold2);
   
   if (result) {
      WriteArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, MASK);
   }
   Exit(result);
   return 0;
}

#endif
